<?php

class ArrayPieceSquadro implements ArrayAccess, Countable {
    private $pieces = [];

    public function offsetExists(mixed $offset): bool {
        return isset($this->pieces[$offset]);
    }

    public function offsetGet(mixed $offset): mixed {
        if (!$this->offsetExists($offset)) {
            throw new OutOfBoundsException("Offset invalide.");
        }
        return $this->pieces[$offset];
    }

    public function offsetSet(mixed $offset, mixed $value): void {
        if (!$value instanceof PieceSquadro) {
            throw new InvalidArgumentException("Value must be an instance of PieceSquadro.");
        }
        if (is_null($offset)) {
            $this->pieces[] = $value;
        } else {
            $this->pieces[$offset] = $value;
        }
    }

    public function offsetUnset(mixed $offset): void {
        if (!$this->offsetExists($offset)) {
            throw new OutOfBoundsException("Offset invalide.");
        }
        unset($this->pieces[$offset]);
        $this->pieces = array_values($this->pieces); // Réindexe le tableau
    }

    public function count(): int {
        return count($this->pieces);
    }

    public function add(PieceSquadro $piece): void {
        $this->pieces[] = $piece;
    }

    public function remove(int $index): void {
        if (!$this->offsetExists($index)) {
            throw new OutOfBoundsException("Index invalide.");
        }
        unset($this->pieces[$index]);
        $this->pieces = array_values($this->pieces); // Réindexe le tableau
    }

    public function toJson(): string {
        return json_encode(array_map(function($piece) {
            return $piece->toJson();
        }, $this->pieces));
    }

    public static function fromJson(string $json): self {
        $data = json_decode($json, true);
        if (json_last_error() !== JSON_ERROR_NONE) {
            throw new InvalidArgumentException("JSON invalide.");
        }
        $array = new self();
        foreach ($data as $pieceJson) {
            $array->add(PieceSquadro::fromJson($pieceJson));
        }
        return $array;
    }
}
?>